"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const logLevels_1 = require("./logLevels");
exports.LogLevels = logLevels_1.LogLevels;
/**
 * Provides methods for writing logs to in different log-levels. Logs are written in JSON format to stdout using console.log().
 */
class Logger {
    constructor() {
        /**
         * This governs which logs will be written. If log level is
         * INFO all logs with INFO and above will be written. So: INFO
         * WARN and ERROR.
         */
        this.level = logLevels_1.LogLevels.INFO;
    }
    /**
     * Prints an enriched log as JSON to stdout with newline.
     * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
     */
    debug(logInput) {
        let debugLevel = logLevels_1.LogLevels.DEBUG;
        if (debugLevel >= this.level) {
            let log = this.createLogJson(logInput, logLevels_1.LogLevels.DEBUG);
            console.log(JSON.stringify(log, null, 2));
        }
    }
    /**
     * Prints an enriched log as JSON to stdout with newline.
     * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
     */
    info(logInput) {
        let infoLevel = logLevels_1.LogLevels.INFO;
        if (infoLevel >= this.level) {
            let log = this.createLogJson(logInput, logLevels_1.LogLevels.INFO);
            console.log(JSON.stringify(log, null, 2));
        }
    }
    /**
     * Prints an enriched log as JSON to stdout with newline.
     * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
     */
    warn(logInput) {
        let warnLevel = logLevels_1.LogLevels.WARN;
        if (warnLevel >= this.level) {
            let log = this.createLogJson(logInput, logLevels_1.LogLevels.WARN);
            console.log(JSON.stringify(log, null, 2));
        }
    }
    /**
     * Prints an enriched log as JSON to stdout with newline.
     * @param logInput {
     * message: 'This is my message',
     * context: AwsLambdaContext,
     * xRequestId: 'id-from-external-system',
     * something: 'my property',
     * somethingElse: 'another extra property' }
     */
    error(logInput) {
        let errorLevel = logLevels_1.LogLevels.ERROR;
        if (errorLevel >= this.level) {
            logInput = this.preProcessErrorProperties(logInput);
            let log = this.createLogJson(logInput, logLevels_1.LogLevels.ERROR);
            console.log(JSON.stringify(log, null, 2));
        }
    }
    createLogJson(logInput, level) {
        // @ts-ignore
        let logOutput = {};
        let context = this.context;
        logOutput['@timestamp'] = Date.now();
        logOutput.level = logLevels_1.LogLevels[level];
        logOutput.message = logInput.message;
        /**
         * By using the lambda context we can easily add more rich logging.
         */
        if (context) {
            logOutput.awsAccount = context.invokedFunctionArn ? context.invokedFunctionArn.split(':')[4] : undefined;
            logOutput.awsRequestId = context.awsRequestId ? context.awsRequestId : undefined;
            logOutput.functionName = context.functionName ? context.functionName : undefined;
            logOutput.functionVersion = context.functionVersion ? context.functionVersion : undefined;
            logOutput.invokedFunctionArn = context.invokedFunctionArn ? context.invokedFunctionArn : undefined;
            logOutput.logGroupName = context.logGroupName ? context.logGroupName : undefined;
            logOutput.logStreamName = context.logStreamName ? context.logStreamName : undefined;
            logOutput.memoryLimitInMB = context.memoryLimitInMB ? context.memoryLimitInMB : undefined;
            logOutput.remainingTime = context.getRemainingTimeInMillis ? context.getRemainingTimeInMillis() : undefined;
        }
        for (let prop of Object.keys(logInput)) {
            if (prop === 'message' || prop === 'xRequestId' || prop === 'context') {
                continue;
            }
            logOutput[prop] = logInput[prop];
        }
        return logOutput;
    }
    /**
     * If we find a property of type Error we cannot stringify it directly because it
     * will just be outputted as "{}", instead if we find an instanceof Error we add
     * the stack to a stack property.
     */
    preProcessErrorProperties(logInput) {
        for (let prop of Object.keys(logInput)) {
            if (logInput[prop] instanceof Error) {
                logInput[prop] = logInput[prop].stack;
            }
        }
        return logInput;
    }
}
const log = new Logger();
exports.log = log;
//# sourceMappingURL=logger.js.map