# A nodejs logger for AWS Lambda
* Writes logs to stdout using console.log
* Logs in JSON format
* Supports log levels: DEBUG, INFO, WARN, ERROR and OFF
* Defaults to log level INFO
* Built using typescript and includes types

## Basic example:
```typescript
import { APIGatewayEvent } from 'aws-lambda';
import { log } from 'pn-lambda-logger';
import { ok } from '../aws/response';

export const healthCheck = async (event: APIGatewayEvent, context: AWSLambda.Context, cb: AWSLambda.Callback) => {

// Setup so that the AWS Lambda meta data will be added to log output
log.context = context;

// A standard INFO log message
log.info({ message: 'Message' });

}
```
```typescript
  // Output
  {
    "@timestamp": 1541421856289,
    "level": "INFO",
    "message": "Message"
  }
```

## Context and meta data
```typescript
import { APIGatewayEvent } from 'aws-lambda';
import { log } from 'pn-lambda-logger';
import { ok } from '../aws/response';

export const healthCheck = async (event: APIGatewayEvent, context: AWSLambda.Context, cb: AWSLambda.Callback) => {

  // Setup so that the AWS Lambda meta data will be added to log output
  log.context = context;

  // A standard INFO log message
  log.info({ message: 'Message' });

  cb(null, ok({ status: 'UP' }));
};
```
```JSON
  {
  "@timestamp": 1541421856289,
  "level": "INFO",
  "message": "Message",
  "awsAccount": "123456789012",
  "awsRequestId": "a3de505e-f16b-42f4-b3e6-bcd2e4a73903",
  "functionName": "ExampleCloudFormationStackName-ExampleLambdaFunctionResourceName-AULC3LB8Q02F",
  "functionVersion": "$LATEST",
  "invokedFunctionArn": "arn:aws:lambda:us-west-2:123456789012:function:ExampleCloudFormationStackName-ExampleLambdaFunctionResourceName-AULC3LB8Q02F",
  "logGroupName": "/aws/lambda/ExampleCloudFormationStackName-ExampleLambdaFunctionResourceName-AULC3LB8Q02F",
  "logStreamName": "2015/10/26/[$LATEST]c71058d852474b9895a0f221f73402ad",
  "memoryLimitInMB": 128,
  "remainingTime": 3
}
  ```


## Log Levels 
The default log level is INFO `log.level = LogLevels.INFO;`. <br />
To show debug messages you need to explicitly set the log.level to DEBUG like this:

```typescript
// Set loglevel
log.level = LogLevels.DEBUG;
// Write debug log
log.debug({ message: 'message' });
```
```JSON
{
  "@timestamp": 1541421856291,
  "level": "DEBUG",
  "message": "Message",
}
```
<br />

## Logging extra data
You can add any number of extra properties to the object when logging, like this:
```typescript
// Setup so that the AWS Lambda meta data will be added to log output
  log.context = context;

// Write debug log
log.info({ message: 'Message', myProp: 'hello', myNestedProp: {subProp: 'something'} });
```
```JSON
{
stdoutHijacker.ts:12
  "@timestamp": 1541429192677,
  "level": "INFO",
  "message": "Message",
  "awsAccount": "123456789012",
  "awsRequestId": "a3de505e-f16b-42f4-b3e6-bcd2e4a73903",
  "functionName": "ExampleCloudFormationStackName-ExampleLambdaFunctionResourceName-AULC3LB8Q02F",
  "functionVersion": "$LATEST",
  "invokedFunctionArn": "arn:aws:lambda:us-west-2:123456789012:function:ExampleCloudFormationStackName-ExampleLambdaFunctionResourceName-AULC3LB8Q02F",
  "logGroupName": "/aws/lambda/ExampleCloudFormationStackName-ExampleLambdaFunctionResourceName-AULC3LB8Q02F",
  "logStreamName": "2015/10/26/[$LATEST]c71058d852474b9895a0f221f73402ad",
  "memoryLimitInMB": 128,
  "remainingTime": 3,
  "myProp": "hello",
  "myNestedProp": {
    "subProp": "something"
  }
}
```

## Mute all logs:
```typescript
  log.level = LogLevels.OFF;
```